<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\TeknisaCrud;
use Teknisa\SM\Service\EnderecoParceiroService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class EnderecoParceiroController extends TeknisaCrud {

    protected $crudService;
    protected $dataSourceName = 'enderecoparc_endereco';
    protected $enderecoParceiroService;
    protected $parameterBagColumns = array("P_NRPARCNEGOCIO", "P_NRORG", "P_NRORGPADRAO");

    public function __construct($crudService, ParameterBag $parameterBag, EnderecoParceiroService $enderecoParceiroService) {
        $this->crudService = $crudService;
        $this->enderecoParceiroService = $enderecoParceiroService;
        parent::__construct($crudService, $parameterBag);
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $nrenderecoparc = $this->enderecoParceiroService->saveEnderecoParceiro($row);
            if ($row['__is_new']) {
                $response->addDataset(new DataSet('columnsToReturn', array("NRENDERECOPARC" => $nrenderecoparc)));
            }
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->enderecoParceiroService->removeEnderecoParceiro($row);
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }

    }
}