<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Util\Procedure\FctAtualizaDocumento;
use Teknisa\SM\Factory\Sm00001001DocumentoPropostaFactory;
use Teknisa\SM\Util\Utilities;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Teknisa\SM\Service\SmDocumentosService;

class Sm00001001DocumentoPropostaService {
    
    protected $environment;
    protected $connection;
    protected $documentoPropostaFactory;
    protected $fileServerBaseUrl;
    protected $zhFileUploader;
    protected $smDocumentoService;
    
    public function __construct(EntityManager $entityManager, Environment $environment, Sm00001001DocumentoPropostaFactory $documentoPropostaFactory, $fileServerBaseUrl, Server $zhFileUploader, SmDocumentosService $smDocumentoService) {
        $this->connection               = $entityManager->getConnection();    
        $this->environment              = $environment;
        $this->documentoPropostaFactory = $documentoPropostaFactory;
        $this->fileServerBaseUrl        = $fileServerBaseUrl;
        $this->zhFileUploader           = $zhFileUploader;
        $this->smDocumentoService       = $smDocumentoService;
    }
    
    /**
     * Atualiza o documento de acordo com o modelo encontrado.
     */
    public function atualizaDocumento($nrseqorcadoc, $nrseqmoddocfct) {
        $this->connection->beginTransaction();
        try {
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            
            $documentos = $this->getDocumentoProposta($nrseqorcadoc, $nrorg);
            foreach($documentos as $documento) {
                if (!empty($documento["DSCAMINHOARQ"]))
                    $this->zhFileUploader->deleteFile($documento["DSCAMINHOARQ"]);
            }

            $this->limparAnexoDaProposta($nrseqorcadoc, $nrorg);
            
            $procedure = new FctAtualizaDocumento($this->connection);
            $procedure->executar($nrseqorcadoc, "P", $nrseqmoddocfct, $nrorg, $operador);
            
            $documentos = $this->getDocumentoProposta($nrseqorcadoc, $nrorg);
            foreach($documentos as $documento) {
                $nomeArquivo   = $documento["NMARQUIVO"];
                $tipo          = $documento["DSTIPO"];
                $tamanhoBytes  = $documento["VRTAMANHO"] * 1024;
                $urlFileServer = $documento["DSCAMINHOARQ"];
                
                //$conteudo = @file_get_contents($this->fileServerBaseUrl . $urlFileServer);
                $conteudo = $this->smDocumentoService->getDocumentoFileServer($this->fileServerBaseUrl . $urlFileServer, $nrorg);
                if ($conteudo !== false) {
                
                    $file = Utilities::uploadFileServer($this->zhFileUploader, "anexo_documento_proposta", $nomeArquivo, $conteudo, $tamanhoBytes, $tipo);
                    $caminhoNovo = $file["path"];
                    
                    $params = array(
                        "DSCAMINHOARQ" => $caminhoNovo,
                        "CDOPERULTATU" => $operador,
                        "NRORGULTATU"  => $nrorg,
                        "NRSEQORCADOC" => $nrseqorcadoc,
                        "NRORG"        => $nrorg
                    );
                    
                    $this->documentoPropostaFactory->atualizarCaminhoDocumentoProposta($params);
                }
                
            }
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function limparAnexoDaProposta($nrseqorcadoc, $nrorg) {
        $params = array(
            "NRSEQORCADOC" => $nrseqorcadoc,
            "NRORG"        => $nrorg
        );
        
        $this->documentoPropostaFactory->limparAnexoDaProposta($params);
    }
    
    /**
     * Retorna o registro da tabela de documento da proposta.
     */
    public function getDocumentoProposta($nrseqorcadoc, $nrorg) {
        $params = array(
            "NRSEQORCADOC" => $nrseqorcadoc,
            "NRORG"        => $nrorg
        );
        
        $dataset = $this->documentoPropostaFactory->getDocumentoProposta($params);
        return $dataset;
    }
    /**
     * Valida se a situação de proposta que está sendo inserida 
     * já possui documento
    **/
    public function validarSituacaoDocumento($idSituacao, $nrseqorcamento, $nrorg) {
        try{
            if($idSituacao) {
                $idSituacao = implode("','", $idSituacao);
                $params = array(
                    "NRSEQORCAMENTO"      => $nrseqorcamento,
                    "NRORG"               => $nrorg
                );
                
                $existe = $this->documentoPropostaFactory->getDocumentoPropostaPorSituacao($idSituacao, $params);
                if($existe[0]["IDSITUACAOORCAMENTO"]) {
                    $situacoes = "SITUACAO_JA_CADASTRADA::" . $existe[0]["IDSITUACAOORCAMENTO"];
                    $situacoes = str_replace('ELABORACAO',  'Em Elaboração',     $situacoes);
                    $situacoes = str_replace('CONCLUIDO',   'Concluída',         $situacoes);
                    $situacoes = str_replace('ACEITO',      'Aceita',            $situacoes);
                    $situacoes = str_replace('CANCELADO',   'Cancelada',         $situacoes);
                    $situacoes = str_replace('ENVIADO',     'Documento Enviado', $situacoes);
                    throw new \Exception($situacoes);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        } 
    }

    /**
     * Salva a edição dos modelos de documento
    **/
    public function salvarDocumentoProposta($row, $nrorg, $operador) {
        $nrseqorcadoc = '_' . $row["NRSEQORCADOC"] . '_';

        $registroAtual = $this->getDocumentoProposta($nrseqorcadoc, $nrorg);
        if (count($registroAtual) > 0 && !empty($registroAtual[0]["DSCAMINHOARQ"])) {
            $this->atualizarTamanhoLogotipoDocumentos($nrseqorcadoc, $row["VRTAMANHOLOGO"], $nrorg, $operador);
        }
        else {
            $this->alterarDocumentoProposta($row, $nrorg, $operador);
        }
    }

    public function atualizarTamanhoLogotipoDocumentos($nrseqorcadoc, $vrtamanhologo, $nrorg, $operador) {
        $params = array(
            "VRTAMANHOLOGO" => $vrtamanhologo,
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $operador,
            "NRSEQORCADOC"  => $nrseqorcadoc,
            "NRORG"         => $nrorg
        );

        $this->documentoPropostaFactory->atualizarTamanhoLogotipoDocumentos($params);
    }

    /**
     * Altera o documento da proposta
    **/
    public function alterarDocumentoProposta($row, $nrorg, $operador) {
        $nrseqorcadoc = '_' . $row["NRSEQORCADOC"] . '_';
        $params = array(
            "NMARQUIVO"       => $row["NMARQUIVO"],
            "DSCAMINHOARQ"    => $row["DSCAMINHOARQ"],
            "DSTIPO"          => $row["DSTIPO"],
            "VRTAMANHO"       => $row["VRTAMANHO"],
            "VRTAMANHOLOGO"   => $row["VRTAMANHOLOGO"],
            "OPERADOR"        => $operador,
            "NRORG"           => $nrorg,
            "NRSEQORCADOC"    => $nrseqorcadoc
        );
        $this->documentoPropostaFactory->alterarDocumentoProposta($params);
    }
    
    /**
     * Baixa do fileServe o arquivo de documento e salva em uma pasta temporária
    **/
    public function downloadDocFileServer($nrseqorcadoc, $nrorg) {
        $result    = array();
        $documento = $this->getDocumentoProposta($nrseqorcadoc, $nrorg) ;
        if(count($documento) > 0) {
            $microtime = microtime(true);
            $microtime = str_replace(".", "_", $microtime);
            
            $documento = $documento[0];
            $fileDir   = $this->fileServerBaseUrl . $documento["DSCAMINHOARQ"];
            $fileName  = $microtime . "_" . str_replace(",", "_", $documento["NMARQUIVO"]);
            
            $fileDirTemp = $this->smDocumentoService->downloadDocFileServer($fileDir, $fileName, $nrorg);
            $fileName    = base64_encode($fileName);
            $result = "/downloadArquivo?requestType=Row&row[FILENAME]=" . $fileName . "&row[FILETYPE]=" . $documento["DSTIPO"];
        }
        
        return $result;
    }
    
    public function verificaSeExisteDocumentoPorPropostaESituacao($nrseqorcamento, $idsituacaoorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "IDSITUACAOORCAMENTO" => $idsituacaoorcamento,
            "NRORG"               => $nrorg
        );
        $dataset = $this->documentoPropostaFactory->getQuantidadeDocumentosPorPropostaESituacao($params);
        return ((int)$dataset[0]["QTD"] > 0);
    }
}

?>